@@ -1,27 +1,7 @@
-/* PipeWire
- *
- * Copyright © 2020 Collabora Ltd.
- *   @author George Kiagiadakis <george.kiagiadakis@collabora.com>
- *
- * Permission is hereby granted, free of charge, to any person obtaining a
- * copy of this software and associated documentation files (the "Software"),
- * to deal in the Software without restriction, including without limitation
- * the rights to use, copy, modify, merge, publish, distribute, sublicense,
- * and/or sell copies of the Software, and to permit persons to whom the
- * Software is furnished to do so, subject to the following conditions:
- *
- * The above copyright notice and this permission notice (including the next
- * paragraph) shall be included in all copies or substantial portions of the
- * Software.
- *
- * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
- * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
- * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
- * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
- * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
- * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
- * DEALINGS IN THE SOFTWARE.
- */
+/* PipeWire */
+/* SPDX-FileCopyrightText: Copyright © 2020 Collabora Ltd. */
+/*                         @author George Kiagiadakis <george.kiagiadakis@collabora.com> */
+/* SPDX-License-Identifier: MIT */
 
 #include <pipewire/impl.h>
 #include <pipewire/extensions/session-manager.h>
@@ -30,6 +10,7 @@
 #include <spa/utils/result.h>
 #include <spa/pod/builder.h>
 #include <spa/pod/filter.h>
+#include <spa/pod/dynamic.h>
 
 #define MAX_PARAMS 32
 
@@ -104,8 +85,8 @@ static int method_enum_params(void *object, int seq,
 	struct param_data *pdata;
 	struct spa_pod *result;
 	struct spa_pod *param;
-	uint8_t buffer[1024];
-	struct spa_pod_builder b = { 0 };
+	uint8_t buffer[2048];
+	struct spa_pod_dynamic_builder b = { 0 };
 	uint32_t index;
 	uint32_t next = start;
 	uint32_t count = 0;
@@ -123,15 +104,15 @@ static int method_enum_params(void *object, int seq,
 
 			param = *pw_array_get_unchecked(&pdata->params, index, struct spa_pod*);
 
-			spa_pod_builder_init(&b, buffer, sizeof(buffer));
-			if (spa_pod_filter(&b, &result, param, filter) != 0)
-				continue;
+			spa_pod_dynamic_builder_init(&b, buffer, sizeof(buffer), 4096);
+			if (spa_pod_filter(&b.b, &result, param, filter) == 0) {
+				pw_log_debug(NAME" %p: %d param %u", impl, seq, index);
+				pw_endpoint_stream_resource_param(d->resource, seq, id, index, next, result);
+				count++;
+			}
+			spa_pod_dynamic_builder_clean(&b);
 
-			pw_log_debug(NAME" %p: %d param %u", impl, seq, index);
-
-			pw_endpoint_stream_resource_param(d->resource, seq, id, index, next, result);
-
-			if (++count == num)
+			if (count == num)
 				return 0;
 		}
 	}
