@@ -1,26 +1,6 @@
-/* PipeWire
- *
- * Copyright © 2018 Wim Taymans
- *
- * Permission is hereby granted, free of charge, to any person obtaining a
- * copy of this software and associated documentation files (the "Software"),
- * to deal in the Software without restriction, including without limitation
- * the rights to use, copy, modify, merge, publish, distribute, sublicense,
- * and/or sell copies of the Software, and to permit persons to whom the
- * Software is furnished to do so, subject to the following conditions:
- *
- * The above copyright notice and this permission notice (including the next
- * paragraph) shall be included in all copies or substantial portions of the
- * Software.
- *
- * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
- * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
- * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
- * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
- * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
- * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
- * DEALINGS IN THE SOFTWARE.
- */
+/* PipeWire */
+/* SPDX-FileCopyrightText: Copyright © 2018 Wim Taymans */
+/* SPDX-License-Identifier: MIT */
 
 #include "config.h"
 
@@ -47,6 +27,7 @@
 
 #include "pipewire.h"
 #include "private.h"
+#include "i18n.h"
 
 #define MAX_SUPPORT	32
 
@@ -519,7 +500,6 @@ parse_pw_debug_env(void)
 	const char *str;
 	char **tokens;
 	int n_tokens;
-	size_t slen;
 	char json[1024] = {0};
 	char *pos = json;
 	char *end = pos + sizeof(json) - 1;
@@ -527,7 +507,7 @@ parse_pw_debug_env(void)
 
 	str = getenv("PIPEWIRE_DEBUG");
 
-	if (!str || (slen = strlen(str)) == 0)
+	if (!str || !*str)
 		return NULL;
 
 	/* String format is PIPEWIRE_DEBUG=[<glob>:]<level>,...,
@@ -541,12 +521,11 @@ parse_pw_debug_env(void)
 		int i;
 		for (i = 0; i < n_tokens; i++) {
 			int n_tok;
-			char **tok;
-			char *pattern;
+			char *tok[2];
 
-			tok = pw_split_strv(tokens[i], ":", 2, &n_tok);
+			n_tok = pw_split_ip(tokens[i], ":", SPA_N_ELEMENTS(tok), tok);
 			if (n_tok == 2 && parse_log_level(tok[1], &lvl)) {
-				pattern = tok[0];
+				char *pattern = tok[0];
 				pos += spa_scnprintf(pos, end - pos, "{ %s = %d },",
 						     pattern, lvl);
 			} else if (n_tok == 1 && parse_log_level(tok[0], &lvl)) {
@@ -555,8 +534,6 @@ parse_pw_debug_env(void)
 				pw_log_warn("Ignoring invalid format in PIPEWIRE_DEBUG: '%s'",
 						tokens[i]);
 			}
-
-			pw_free_strv(tok);
 		}
 	}
 	pw_free_strv(tokens);
@@ -589,6 +566,8 @@ void pw_init(int *argc, char **argv[])
 	if (support->init_count > 0)
 		goto done;
 
+	pw_random_init();
+
 	pthread_mutex_lock(&support_lock);
 	support->in_valgrind = RUNNING_ON_VALGRIND;
 
